/********************************************************************************
 *                                  DISCLAIMER                                  *
 * This software and documentation (the "Software") were developed at the Food  *
 * and Drug Administration (FDA) by employees of the Federal Government in the  *
 * course of their official duties. Pursuant to Title 17, Section 105 of the    *
 * United States Code, this work is not subject to copyright protection and is  *
 * in the public domain. Permission is hereby granted, free of charge, to any   *
 * person obtaining a copy of the Software, to deal in the Software without     *
 * restriction, including without limitation the rights to use, copy, modify,   *
 * merge, publish, distribute, sublicense, or sell copies of the Software or    *
 * derivatives, and to permit persons to whom the Software is furnished to do   *
 * so. FDA assumes no responsibility whatsoever for use by other parties of     *
 * the Software, its source code, documentation or compiled executables, and    *
 * makes no guarantees, expressed or implied, about its quality, reliability,   *
 *  or any other characteristic. Further, use of this code in no way implies    *
 * endorsement by the FDA or confers any advantage in regulatory decisions.     *
 * Although this software can be redistributed and/or modified freely, we       *
 * ask that any derivative works bear some notice that they are derived from    *
 * it, and any modified versions bear some notice that they have been modified. * 
 *                                                                              *
 * Written by:                                                                  *
 *  Lars Johannesen                                                             *
********************************************************************************/
#include "mex.h"

#include <armadillo>

template<typename TYP>
TYP get_matlab_scalar(const mxArray *a) {
	TYP *t = (TYP*)mxGetData(a);
	return *t;
}

bool isDoubleScalar(const mxArray *in) {
        return mxIsDouble(in) && mxGetNumberOfElements(in)==1;
}

struct peak {
	int idx;
	int left;
	int right;
	bool good;
	int cpeak;
};

typedef std::vector<peak> peaks;

peak find_cog(const arma::mat &sig, const int cpeak) {
	peak p;

	const int nsig = sig.n_elem;
	
	// Fraction used for determining left/right for both area and slope based
	const double frac = 0.5;
	bool good = false;

	// Threshold used for finding left/right of peak based on amplitude alone
	// the onset/offset will be redefined later using slopes
	double thr = sig(cpeak) * frac;

	// Step1a: Determine the first value below frac of cpeak (onset)
	//         Also compute the area to the left (for center of gravitiy location)
	int valleft = cpeak;
	double leftsum = 0;
	while(valleft > 0) {
		if(sig(valleft) < thr) break;
	
		--valleft;
		leftsum += sig(valleft);
	}

	// Step1b: Determine the first value below frac of cpeak (offset)
	//         Also compute the area to the right (for center of gravitiy location)
	int valright = cpeak;
	double rightsum = 0;
	while(valright < (nsig-1)) {
		if(sig(valright) < thr) break;

		++valright;
		rightsum += sig(valright);
	}

	// Step2: Locate center of gravity
	double sum = leftsum + rightsum + sig(cpeak);
	double peakthr = sum / 2.;
	int idx = valleft;

	double csum = 0;
	while(idx < valright) {
		csum += sig(idx);

		if (csum > peakthr) break;

		++idx;
	}

	// Assign output
	p.cpeak = cpeak;
	p.left = valleft;
	p.right = valright;
	p.good = true;
	p.idx = idx;

	return(p);
}

peaks detect_peaks(const arma::mat &sig, const double fs) {
	// Minimum RR, used for refactoriness that is two detections cannot occur within minrr
	const int minrr = static_cast<int>(round(250 * fs/1000.));

	// Current candidate peak
	int cpeak = 0;

	// Detection threshold see description for ready below
	const double frac = 0.5;

	// Are we ready to detect? That is have we moved down to a point at least frac below cpeak?
	bool ready = 0;

	const int nsig = sig.n_elem;

	peaks p;
	int i = 0;

	while (i < nsig) {
		// Case1: We are still going up relative to the current candidate
		//        - Set candidate to current
		//        - Set ready to false as we are still in a peak
		if(sig(i) > sig(cpeak)) {
			cpeak = i;
			ready = false;
		// Case2: We have passed down sufficiently far that we got a peak
		//        - If we have not set ready set ready flag
		} else if(sig(i) < sig(cpeak)*frac && !ready) {
			ready = true;
		// Case3: We are ready (that is we have moved down sufficiently from last cpeak)
		//	  and we have moved far enough in time. This last condition helps ensure
		//	  that we will never detect twice
		} else if(ready && i-cpeak > minrr) {
			// Find center of gravity
			peak cp = find_cog(sig, cpeak);

			// Determine if this peak is really the same as the last
			// for some peaks they are poorly defined in the sense that when applying
			// the center of gravitiy technique to a peak it will get assigned close or
			// at the same location as last peak
			bool overlap = false;

			// So if we have more than one peak detected
			if(p.size() > 0) {
				// Determine if we are close to the last peak
				int lastpeak = p[p.size()-1].idx;
				if (abs(lastpeak-cp.idx) < minrr) {
					// If we are within minrr lets not detect this one
					overlap = true;
				}
			}

			if(!overlap) {
				// Store it
				p.push_back(cp);
			} else {
				if (sig(p[p.size()-1].idx) < sig(cp.idx)) {
					p[p.size()-1] = cp;
				}
			}

			// Set ready to false
			ready = false;
			// Set current candidate to current position
			cpeak = i;
		} // End of if

		// Next sample
		++i;
	} // End of while i < nsig

	return p;
}

void mexFunction (int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	if(nrhs != 2) 
		mexErrMsgTxt("Need 2 arguments data, fs");

	using namespace arma;
	try {
		if(!mxIsDouble(prhs[0]))
			mexErrMsgTxt("First argument must be a double scalar");

		if(!isDoubleScalar(prhs[1]))
			mexErrMsgTxt("Second argument must be a double scalar");

		const double *datptr = (const double*) mxGetPr(prhs[0]);
		int m = mxGetN(prhs[0]);
		int n = mxGetM(prhs[0]);
		mat indat(datptr, n, m);

		double fs = get_matlab_scalar<double>(prhs[1]);

		peaks p = detect_peaks(indat, fs);				
		
		mwSize dims[2];
		dims[0] = p.size();
		dims[1] = 5;
		mxArray *a = mxCreateNumericArray(2, dims, mxDOUBLE_CLASS, mxREAL);
		double *dst = (double*)mxGetData(a);
		for(int i = 0; i < p.size(); ++i) {
			dst[i + (0 * p.size())] = p[i].idx;
			dst[i + (1 * p.size())] = p[i].left;
			dst[i + (2 * p.size())] = p[i].right;
			dst[i + (3 * p.size())] = p[i].good == true ? 1 : 0;
			dst[i + (4 * p.size())] = p[i].cpeak;
		}

		plhs[0] = a;
	} catch (std::logic_error &exp) {
		mexErrMsgTxt(exp.what());
	} catch (...) {
		mexErrMsgTxt("Unknown error");
	}
}
