%% @file
% ecg_detector.m
%
% @author Lars Johannesen
%
% @section DISCLAIMER                                  
% This software and documentation (the "Software") were developed at the Food  
% and Drug Administration (FDA) by employees of the Federal Government in the  
% course of their official duties. Pursuant to Title 17, Section 105 of the    
% United States Code, this work is not subject to copyright protection and is  
% in the public domain. Permission is hereby granted, free of charge, to any   
% person obtaining a copy of the Software, to deal in the Software without     
% restriction, including without limitation the rights to use, copy, modify,   
% merge, publish, distribute, sublicense, or sell copies of the Software or    
% derivatives, and to permit persons to whom the Software is furnished to do   
% so. FDA assumes no responsibility whatsoever for use by other parties of     
% the Software, its source code, documentation or compiled executables, and    
% makes no guarantees, expressed or implied, about its quality, reliability,   
%  or any other characteristic. Further, use of this code in no way implies    
% endorsement by the FDA or confers any advantage in regulatory decisions.     
% Although this software can be redistributed and/or modified freely, we       
% ask that any derivative works bear some notice that they are derived from    
% it, and any modified versions bear some notice that they have been modified. 
%
% @section Examples:
%  [idx] = ecg_detector(ecg,0);
%
% @section See also:
% qrsdetect
% qrsondetect

%% 
% @brief Detects the beats in the ecg
% @param ecg Vector with the ecg signal sampled at 250Hz
% @param plotflag Reserved (for debug purposes, not use in the current version)
% @return anns Vector of annotations with the detected beats in ms
%
function [anns,sqi,aux] = ecg_detector(ecg, plotflag)
%ecg_detector Detects the beats in the ecg 
%
% ecg_detector(ecg)
%
% ecg - vector (250 Hz)
%
%                                  DISCLAIMER                                  
% This software and documentation (the "Software") were developed at the Food  
% and Drug Administration (FDA) by employees of the Federal Government in the  
% course of their official duties. Pursuant to Title 17, Section 105 of the    
% United States Code, this work is not subject to copyright protection and is  
% in the public domain. Permission is hereby granted, free of charge, to any   
% person obtaining a copy of the Software, to deal in the Software without     
% restriction, including without limitation the rights to use, copy, modify,   
% merge, publish, distribute, sublicense, or sell copies of the Software or    
% derivatives, and to permit persons to whom the Software is furnished to do   
% so. FDA assumes no responsibility whatsoever for use by other parties of     
% the Software, its source code, documentation or compiled executables, and    
% makes no guarantees, expressed or implied, about its quality, reliability,   
%  or any other characteristic. Further, use of this code in no way implies    
% endorsement by the FDA or confers any advantage in regulatory decisions.     
% Although this software can be redistributed and/or modified freely, we       
% ask that any derivative works bear some notice that they are derived from    
% it, and any modified versions bear some notice that they have been modified. 
%                                                                              
% Written by: Lars Johannesen
%
% Examples:
%  [idx,sqi] = ecg_detector(ecg);
%
% See also:
% qrsdetect
% qrsondetect
	Fs = 250;

	% ECG detection
	[b,a] = butter(5,[5 25]/(Fs/2.));
	ecg2 = filtfilt(b,a,ecg);

	cfg = qrsdetect();
	cfg.minrr = 150;
	cfg.slopefrac = 0.5;
	cfg.training_time = 60;
	cfg.thrdegrad=0.5;
	cfg.maxdet = 100.;
	cfg.maxsigchg = 100.;
	cfg.maxnoisechg = 100.;
	cfg.maxwidth = 300;
	cfg.minsnr = 0;
	cfg.frac = 0.25;
	cfg.detthr1 = 0.15;
	cfg.maxinitmissed = 100;
	cfg.maxthrlower= 0.8;
	cfg.detfrac = 0.5;
	cfg.u3window = 104;
	cfg.ith = 4;
	u3 = calcu3(ecg2, Fs, cfg.u3window, cfg.ith);
	[anns,dbg,cfg] = qrsdetect(u3, Fs, cfg); % annotations are expected to be in samples  

	maxdiff = 300 * Fs/1000.;
	minsnr = 1;
	anns = det_tall_t(u3, anns, Fs);
	anns = anns(:);
	sqi = ones(length(anns),1);
	%sqi = ecg_sqi(anns, dbg, maxdiff, minsnr);

	if (plotflag == 1)
		u3max = max(u3);
		u3 = u3/u3max;
		ecg = ecg/max(ecg);

		plot(u3,'b');
		hold on;
		plot(ecg-mean(ecg),'r');
		plot(dbg{1}(:,1),dbg{1}(:,2)/u3max,'g-');
		plot(dbg{1}(:,1),dbg{1}(:,3)/u3max,'r-');
		plot(dbg{1}(:,1),dbg{1}(:,4)/u3max,'k-');
		plot(dbg{1}(:,1),dbg{1}(:,5)/u3max,'k--');

		idx = dbg{1}(:,7)~=0;
		plot(dbg{1}(idx,7)+1,u3(dbg{1}(idx,7)+1),'k*');
		plot(dbg{1}(idx,8)+1,u3(dbg{1}(idx,8)+1),'g*');
		plot(dbg{1}(idx,9)+1,u3(dbg{1}(idx,9)+1),'r*');

		plot(anns(sqi==1),u3(anns(sqi==1))/u3max,'g*');
		plot(anns(sqi~=1),u3(anns(sqi~=1))/u3max,'r*');
		plot(anns,[0;diff(anns)*4]/1000*0.5,'g');
		ylim([0 5]);
		hold off;
	end

	aux = struct('u3',u3);
	aux.dbg = dbg;
end
