/**
 * @file ecglib.hpp
 * @author Lars Johannesen <lj@halloffame.dk>
 *
 * @section LICENSE
 * This program is licensed under LGPL3, see LICENSE
 * 
 * @section DESCRIPTION
 * The enums in this file describes leads, annotations, etc
 */
#ifndef ECGLIB_ECGLIB_LJ_2013_04_05
#define ECGLIB_ECGLIB_LJ_2013_04_05 1

#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/enum.hpp>
#include <boost/assert.hpp>

#include <armadillo>

#include <algorithm>

#include <vector>
#include <string>

#include <map>
#include <stdexcept>

namespace ecglib {
	using namespace arma;

	struct ecglib_exception : std::logic_error {
		explicit ecglib_exception(const std::string &what) : std::logic_error(what) {
		}
	};

	typedef unsigned int sample;
	typedef unsigned int timems;

	BOOST_ENUM_VALUES(gender, std::string,
		(UNKNOWN)("Unknown")
		(MALE)("Male")
		(FEMALE)("Female")
	);

	BOOST_ENUM_VALUES(annotation_type, std::string,
		(PON)("P-wave onset")
		(PPEAK)("P-wave peak")
		(POFF)("P-wave offset")
		(QON)("QRS onset")
		(QPEAK)("Q-peak")
		(RPEAK)("R-wave peak")
		(RPPEAK)("R'-wave peak")
		(SPEAK)("S-wave peak")
		(QOFF)("QRS offset")
		(TON)("T-wave onset")
		(TPEAK)("T-wave peak")
		(TPPEAK)("T-wave second peak")
		(TOFF)("T-wave offset")
		(UON)("U-wave onset")
		(UPEAK)("U-wave peak")
		(UOFF)("U-wave offset")
	);

	BOOST_ENUM_VALUES(annotation_subtype, std::string,
		(NONE)("None")
		(NORMAL)("Normal")
		(VPC)("Ventricular premature contraction")
		(APB)("Atrial premature beat")
		(LBBB)("Left bundle branch block beat")
		(RBBB)("Right bundle branch block beat")
		(UBBB)("(unspecified) bundle branch block beat")
		(AAPB)("Aberrated atrial premature beat")
		(NJPB)("Nodal (junctional) premature beat")
		(SPB)("Supraventricular premature or ectopic beat (atrial/nodal)")
		(RONT)("R-on-T premature ventricular contraction")
		(FVNB)("Fusion of ventricular and normal beat")
		(AESC)("Atrial escape beat")
		(NJESC)("Nodal (junctional) premature beat")
		(SVESC)("Supraventricular escape beat (atrial or nodal)")
		(VESC)("Ventricular escape beat")
		(PACED)("Paced beat")
		(FPN)("Fusion of paced and normal beat")
		(UNKNOWN)("Unknown")
	);

	BOOST_ENUM_VALUES(ecglead, std::string,
		(GLOBAL)("Global")
		(I)("I")
		(II)("II")
		(III)("III")
		(AVR)("avR")
		(AVF)("avF")
		(AVL)("avL")
		(V1)("V1")
		(V2)("V2")
		(V3)("V3")
		(V4)("V4")
		(V5)("V5")
		(V6)("V6")
		(ES)("ES")
		(AS)("AS")
		(AI)("AI")
		(X)("X")
		(Y)("Y")
		(Z)("Z")
		(VCGMAG)("VCGGMAG")
		(UNKNOWN1)("Unknown 1") 
		(UNKNOWN2)("Unknown 2")
		(UNKNOWN3)("Unknown 3")
		(UNKNOWN4)("Unknown 4")
		(UNKNOWN5)("Unknown 5")
		(UNKNOWN6)("Unknown 6")
		(MLII)("Modified II")
                (AtrialBi)("Atrial-Bi")
		(VentrBi)("Ventr-Bi")
		(AtrialUn)("Atrial-Un")
		(VentrUn)("Ventr-Un")
	);

	struct ecgheader{
		std::string *filename; /*! Filename from where the header was loaded */
		bool skipcrc; /*! True if no CRC present or if CRC skipped while loading */
		double fs; /*! Assuming same sampling frequency for all leads (loaders will need to interpolate up/down as needed) */
		int nsamples; /*! Assuming same number of samples for all leads (loaders will need to interpolate up/down as needed) */
		double resolution[20]; /*! Resolution of each lead*/
		double precision[20]; /*! Precision of each lead*/

		//TODO: Select between either  std::vetor<ecglead> leads or nleads+leadnames.
		int nleads; /*! Number of leads */
		std::string *leadnames[20]; /*! Name of the leads */
		std::vector<ecglead> *leads; /*! Provided for backward compatility with ENUM leads */
		std::string id; /*! Subject's ID */
		boost::gregorian::date birthdate; /*! Subject's birthdate */
		gender sex; /*! Subject's gender */
		boost::posix_time::ptime recordstart;
		boost::posix_time::ptime recordstop;
		//FIXME: add crc field typedef uint16_t 
		long startoffset; /*! offset in bytes where the ecg data starts in the file. To be use by big files (bigfoot) access */
	};
}

#endif
