/**
 * @file qrsdetector.hpp
 * @author Lars Johannesen <lj@halloffame.dk>
 *
 * @section LICENSE
 * This program is licensed under LGPL3, see LICENSE
 * 
 * @section DESCRIPTION
 * The classes in this file are used for describing characteristics about ecg records
 */

#include "qrsdetector.hpp"

namespace ecglib { 
	void qrsdetector_config::defaults() {
		add("detthr1",property(Type::Double,0.15,"Primary detection threshold (Noise + (Signal-Noise)*detthr1"));
		add("detthr2",property(Type::Double,0.5,"Secondary detection threshold (primary * detthr2"));
		add("minrr",property(Type::Int,250,"Refactory period for detections"));
		add("maxval",property(Type::Double,150.,"Maximum U3 value for detections in training"));
		add("minval",property(Type::Double,0.1,"Minimum U3 value for detections in training"));
		add("updfrac",property(Type::Double,1/8.,"Fraction to weigh current signal/noise peak at for updating signal/noise level"));
		add("rrhist",property(Type::Uint,8u,"Number of RR measurements to include in history"));
		add("peakhist",property(Type::Int,20,"Peak history"));
		add("maxmove",property(Type::Int,50,"When detecting center of gravity, how far is the code allowed to move the peak"));
		add("trainfrac",property(Type::Double,0.5,"Training fraction, Est. signal = median(trainpeaks)*trainfrac"));
		add("minwidth",property(Type::Int,50,"Minimum width for QRS (ms)"));
		add("maxwidth",property(Type::Int,250,"Maximum width for QRS (ms)"));
		add("minleft",property(Type::Int,25,"Minimum left width for qrs (ms)"));
		add("minright",property(Type::Int,25,"Minimum right width for qrs (ms)"));
		add("searchwindow",property(Type::Int,50,"Search window for finding peaks when a global detection has been triggered"));
		add("nreq",property(Type::Double,0.5,"Number of leads to require for secondary detection (required = floor(nchannels * nreq))"));
		add("missedtime",property(Type::Int,10000,"No detections for x ms will result in some degree of threshold lowering"));
		add("maxmissed",property(Type::Int,10,"Number of maximum missed, used for threshold lowering"));
		add("maxthrlower",property(Type::Double,0.5,"Maximum threshold lowering, will be linear per missed beat"));
		add("u3window",property(Type::Int,100,"Window width in ms"));
		add("ith",property(Type::Int,4,"The smoothing factor of the u3"));
		add("maxsigchg",property(Type::Double,70.,"Maximum growth/decline in estimated signal value"));
		add("maxnoisechg",property(Type::Double,70.,"Maximum growth/decline in estimated noise value"));
		add("trainingtime",property(Type::Int,60,"Training time in s"));
		add("minsnr",property(Type::Double,0.,"Required fraction to initiate a detection by a lead (sig/noise)"));
		add("maxinitmissed",property(Type::Int,0,"To trigger a global detection both minsnr and maxmissedinit must be satisfied"));
		add("maxdet",property(Type::Double,100.,"Peaks that are maxdet times the signal threshold are ignored."));
		add("frac",property(Type::Double,0.5,"Fraction for finding left/right of U3"));
		add("slopefrac",property(Type::Double,0.25,"Fraction for finding left/right of U3 [slope]"));
		add("detfrac",property(Type::Double,0.5,"Fraction for triggering a candidate peak in U3"));
		add("thrdegrad",property(Type::Double,0.5,"Fraction to degrade signal level when secondary detection occurs"));
		add("detectonoff", property(Type::Int,1,"Save left/right as on/off"));
	}

	qrsdetector_state::qrsdetector_state(const qrsdetector_config &cfg) : _ready(false), _missed(0) {
		_speaks.resize(cfg.get<int>("peakhist"));
		_npeaks.resize(cfg.get<int>("peakhist"));

		_frac1 = cfg.get<double>("updfrac");
		_frac2 = 1.-_frac1;

		_thr1frac = cfg.get<double>("detthr1");
		_thr2frac = cfg.get<double>("detthr2");

		_maxsigchg = cfg.get<double>("maxsigchg");
		_maxnoisechg = cfg.get<double>("maxnoisechg");

		_maxmissed = cfg.get<int>("maxmissed");
		_maxthrlower = cfg.get<double>("maxthrlower");
		_thrpenalty = 1;
	}

	void qrsdetector_state::detect_speak(const std::size_t loc, const double val) {
		qrsdetector_peak np(loc, val);
		_speaks.push_back(np);

		double valnew = val;
		if(_spki > 0 && fabs(1.-valnew/_spki) > _maxsigchg) {
			valnew = valnew > _spki ? (1+_maxsigchg) * _spki : (1-_maxsigchg) * _spki;
		}

		double spki = _frac1 * valnew + _frac2 * _spki;
		_spki = spki;

		if(_npki > thr1()) {
			_npki = _frac1 * thr1() + _frac2 * _npki;
		}

		update_thresholds();
	}

	void qrsdetector_state::detect_npeak(const std::size_t loc, const double val) {
		qrsdetector_peak np(loc, val);
		_npeaks.push_back(np);

		double valnew = val;
		if(_npki > 0 && fabs(1.-valnew/_npki) > _maxnoisechg) {
			valnew = valnew > _npki ? (1+_maxnoisechg) * _npki : (1-_maxnoisechg) * _npki;
		}

		_npki = _frac1 * val + _frac2 * _npki;

		update_thresholds();
	}

	std::size_t qrsdetector_state::max_npeak(const std::size_t begin, const std::size_t end) const {
		std::size_t num = 0;

		bool found = false;
		double maxval = -1;
		std::size_t bestnum = 0;

		for(auto &it : _npeaks) {
			if(it.loc > begin && it.loc < end) {
				if(!found) {
					found = true;
					maxval = it.value;
					bestnum = num;
				} else {
					if(it.value > maxval) {
						maxval = it.value;
						bestnum = num;
					}
				}
			}

			++num;
		}

		if(!found) throw std::logic_error("No peaks found");

		return bestnum;
	}
}
