%% @file
% est_lag.m
%
% @author Lars Johannesen
%
% @section DISCLAIMER
% This software and documentation (the "Software") were developed at the Food
% and Drug Administration (FDA) by employees of the Federal Government in the
% course of their official duties. Pursuant to Title 17, Section 105 of the
% United States Code, this work is not subject to copyright protection and is
% in the public domain. Permission is hereby granted, free of charge, to any
% person obtaining a copy of the Software, to deal in the Software without
% restriction, including without limitation the rights to use, copy, modify,
% merge, publish, distribute, sublicense, or sell copies of the Software or
% derivatives, and to permit persons to whom the Software is furnished to do
% so. FDA assumes no responsibility whatsoever for use by other parties of
% the Software, its source code, documentation or compiled executables, and
% makes no guarantees, expressed or implied, about its quality, reliability,
%  or any other characteristic. Further, use of this code in no way implies
% endorsement by the FDA or confers any advantage in regulatory decisions.
% Although this software can be redistributed and/or modified freely, we
% ask that any derivative works bear some notice that they are derived from
% it, and any modified versions bear some notice that they have been modified.
%
% @section Examples:
%  [lags, aux] = est_lag(out, ref, anns, fs, total_len, wavf, maxlag, lagwindow)

%%
% @brief Determines the lag between ref and num
% @param ref Reference annotations (samples)
% @param num Annotations (samples)
% @param total_len Total length (in samples)
% @param wavf Waveform used to create series
% @param max_lag Maximum lag used in xcorr
% @param lagwindow for inside/outside ratio
% @return [lags, ratio, aux] = Lags as a number in ms, and aux is aux info for debugging and ratio is largest peak inside/outside lagwindow ratio
%
function [lags,ratio,aux] = est_lag(refanns, anns, total_len, wavf, maxlag,lagwindow)
%est_lag Determines lag between ref and number
%
% est_lag(ref, anns, wavf, maxlag, lagwindow)
%
% ref Referenced number in out (samples)
% num Annotations (samples)
% total_len Total length (in samples)
% wavf Waveform used to create series
% min_sqi Minimum sqi to be used
% lagwindow for inside/outside ratio
%
%                                  DISCLAIMER
% This software and documentation (the "Software") were developed at the Food
% and Drug Administration (FDA) by employees of the Federal Government in the
% course of their official duties. Pursuant to Title 17, Section 105 of the
% United States Code, this work is not subject to copyright protection and is
% in the public domain. Permission is hereby granted, free of charge, to any
% person obtaining a copy of the Software, to deal in the Software without
% restriction, including without limitation the rights to use, copy, modify,
% merge, publish, distribute, sublicense, or sell copies of the Software or
% derivatives, and to permit persons to whom the Software is furnished to do
% so. FDA assumes no responsibility whatsoever for use by other parties of
% the Software, its source code, documentation or compiled executables, and
% makes no guarantees, expressed or implied, about its quality, reliability,
%  or any other characteristic. Further, use of this code in no way implies
% endorsement by the FDA or confers any advantage in regulatory decisions.
% Although this software can be redistributed and/or modified freely, we
% ask that any derivative works bear some notice that they are derived from
% it, and any modified versions bear some notice that they have been modified.
%
% Written by: Lars Johannesen + Loriano Galeotti 
%
% Examples:
%  [lags, rat, aux] = est_lag(out{1}.anns, out{2}.anns, total_time, wavf, 3000, [-700 0])
%
% See also:
% 
% Bugfix: Added default value for lagwindow - LG 
	lags = 0;
	aux = cell(1,1);
	ratio = NaN;
	if nargin < 6 || length(lagwindow) < 2 % LG added default lagwindow definition
	    lagwindow = [-700 0];
	end

	ref_ts = make_series(refanns, wavf, total_len);
	oth_ts = make_series(anns,  wavf, total_len);
	[xv, lag] = xcorr(ref_ts, oth_ts,maxlag);
		
	[peakin, idxin] = max(xv(lag>lagwindow(1)&lag<lagwindow(2)));
	[peakout, idxout] = max(xv(lag<lagwindow(1)|lag>lagwindow(2)));
		
	if isempty(peakin)
	    peakin = 0;
	    lags = NaN;
	else
	    lags = lag(idxin+find(lag>lagwindow(1),1,'first'));
	end

	if isempty(peakout) || peakout == 0
	    ratio = Inf;
	else
	    ratio = peakin / peakout;
	end

	aux = struct('xv',xv,'lag',lag,'ref_ts',ref_ts,'oth_ts',oth_ts);
end
