/********************************************************************************
 *                                  DISCLAIMER                                  *
 * This software and documentation (the "Software") were developed at the Food  *
 * and Drug Administration (FDA) by employees of the Federal Government in the  *
 * course of their official duties. Pursuant to Title 17, Section 105 of the    *
 * United States Code, this work is not subject to copyright protection and is  *
 * in the public domain. Permission is hereby granted, free of charge, to any   *
 * person obtaining a copy of the Software, to deal in the Software without     *
 * restriction, including without limitation the rights to use, copy, modify,   *
 * merge, publish, distribute, sublicense, or sell copies of the Software or    *
 * derivatives, and to permit persons to whom the Software is furnished to do   *
 * so. FDA assumes no responsibility whatsoever for use by other parties of     *
 * the Software, its source code, documentation or compiled executables, and    *
 * makes no guarantees, expressed or implied, about its quality, reliability,   *
 *  or any other characteristic. Further, use of this code in no way implies    *
 * endorsement by the FDA or confers any advantage in regulatory decisions.     *
 * Although this software can be redistributed and/or modified freely, we       *
 * ask that any derivative works bear some notice that they are derived from    *
 * it, and any modified versions bear some notice that they have been modified. * 
 *                                                                              *
 * Written by:                                                                  *
 *  Lars Johannesen                                                             *
 ********************************************************************************/
#include "mex.h"

#include <wfdb/wfdb.h>

#include <vector>
#include <map>
#include <string>

#include <iostream>

#include <stdexcept>

#include <armadillo>

mxArray *load_sig(const mxArray *signam, int nsamp, char *start);

template<typename TYP>
TYP get_matlab_scalar(const mxArray *a) {
	double *t = (double*)mxGetData(a);
	return static_cast<TYP>(*t);
}

template<typename TYP>
mxArray *set_matlab_scalar(TYP t, mxClassID cla=mxDOUBLE_CLASS) {
	mwSize dims[2] = {1,1};
	mxArray *o = mxCreateNumericArray(2, dims, cla, mxREAL);
	double *d = (double*)mxGetData(o);
	*d=t;
	return o;
}

bool isDoubleScalar(const mxArray *in) {
	return mxIsDouble(in) && mxGetNumberOfElements(in)==1;
}

void mexFunction (int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
	if(nrhs == 0) {
		mexErrMsgTxt("Need one arguments: record [nsamp] [start]");
	}

	try {
		if(!mxIsChar(prhs[0]))
                        mexErrMsgTxt("First argument must be a char");

		int nsamp = 0;
		if(nrhs >= 2) {
			if(!mxIsDouble(prhs[1])) {
				mexErrMsgTxt("Second argument must be a double");
			}
			
			nsamp = get_matlab_scalar<int>(prhs[1]);
		}

		char *start = NULL;
		if(nrhs >= 3) {
			if(!mxIsChar(prhs[2])) {
				mexErrMsgTxt("Third argument must be a string");
			}
			start = mxArrayToString(prhs[2]);	
		}

		mxArray *sig = load_sig(prhs[0], nsamp, start);
		plhs[0] = sig;
	} catch (...) {
		mexErrMsgTxt("Failed loading");
	}

	wfdbquit();
}

mxArray *load_sig(const mxArray *signam, int nsamp, char *start) {
	char *rec = mxArrayToString(signam);

	WFDB_Siginfo *siarray;
	int nsig = isigopen(rec,NULL,0);

	siarray = (WFDB_Siginfo*)malloc(nsig*sizeof(WFDB_Siginfo));
	nsig = isigopen(rec,siarray,nsig);

	if(nsig == -1) {
		throw std::logic_error("Could not open file");
	}

	mwSize dims[1];
	dims[0] = nsig;
	mxArray *sig = mxCreateCellArray(1, dims);

	double fs = sampfreq(NULL);

	using namespace arma;
	nsamp = nsamp > 0 ? nsamp : siarray[0].nsamp;

	if (start != NULL) {
		WFDB_Time time = strtim(start);
		isigsettime(time);
	}

	mat data = zeros<mat>(nsamp, nsig);
	WFDB_Sample* v = (WFDB_Sample *)malloc(nsig * sizeof(WFDB_Sample));
	std::size_t j = 0;
	for(; j < nsamp; ++j) {
		if(getvec(v) < 0) {
			break;
		}

		for(std::size_t i = 0; i < nsig; ++i) {
			data(j,i) = v[i];
		}
	}

	if (j < nsamp) {
		data.resize(j,nsig);
		nsamp = j;
	}

	const char *sigstruct[] = {"name","rawdata","fs","baseline","gain","units","data","bsize","spf","adcres","adczero","initval"};
	for(int i = 0; i < nsig; ++i) {
		mxArray *cell = mxCreateStructMatrix(1, 1, 12, sigstruct);

		std::string signam(siarray[i].desc);

		mxArray *out = mxCreateDoubleMatrix(nsamp, 1, mxREAL);
		double *dst = (double*) mxGetData(out);
		mxArray *out2 = mxCreateDoubleMatrix(nsamp, 1, mxREAL);
		double *dst2 = (double*) mxGetData(out2);

		for(std::size_t j = 0; j < nsamp; ++j) {
			double samp = data(j,i);

			dst[j] = samp;
			dst2[j] = aduphys(i, samp);
		}

		mxArray *matfs = set_matlab_scalar<double>(fs);
		mxArray *matnam = mxCreateString(siarray[i].desc);
		mxArray *matbas = set_matlab_scalar<double>(siarray[i].baseline);
		mxArray *matgain = set_matlab_scalar<double>(siarray[i].gain);
		mxArray *matunits = mxCreateString(siarray[i].units);
		mxArray *matbsize = set_matlab_scalar<double>(siarray[i].bsize);
		mxArray *matspf = set_matlab_scalar<double>(siarray[i].spf);
		mxArray *matadcres = set_matlab_scalar<double>(siarray[i].adcres);
		mxArray *matadczero = set_matlab_scalar<double>(siarray[i].adczero);
		mxArray *matinitval = set_matlab_scalar<double>(siarray[i].initval);

		mxSetFieldByNumber(cell, 0, 0, matnam);
		mxSetFieldByNumber(cell, 0, 1, out);
		mxSetFieldByNumber(cell, 0, 2, matfs);
		mxSetFieldByNumber(cell, 0, 3, matbas);
		mxSetFieldByNumber(cell, 0, 4, matgain);
		mxSetFieldByNumber(cell, 0, 5, matunits);
		mxSetFieldByNumber(cell, 0, 6, out2);
		mxSetFieldByNumber(cell, 0, 7, matbsize);
		mxSetFieldByNumber(cell, 0, 8, matspf);
		mxSetFieldByNumber(cell, 0, 9, matadcres);
		mxSetFieldByNumber(cell, 0, 10, matadczero);
		mxSetFieldByNumber(cell, 0, 11, matinitval);

		mxSetCell(sig, i, cell);
	}

	return sig;
}
