/********************************************************************************
 *                                  DISCLAIMER                                  *
 * This software and documentation (the "Software") were developed at the Food  *
 * and Drug Administration (FDA) by employees of the Federal Government in the  *
 * course of their official duties. Pursuant to Title 17, Section 105 of the    *
 * United States Code, this work is not subject to copyright protection and is  *
 * in the public domain. Permission is hereby granted, free of charge, to any   *
 * person obtaining a copy of the Software, to deal in the Software without     *
 * restriction, including without limitation the rights to use, copy, modify,   *
 * merge, publish, distribute, sublicense, or sell copies of the Software or    *
 * derivatives, and to permit persons to whom the Software is furnished to do   *
 * so. FDA assumes no responsibility whatsoever for use by other parties of     *
 * the Software, its source code, documentation or compiled executables, and    *
 * makes no guarantees, expressed or implied, about its quality, reliability,   *
 *  or any other characteristic. Further, use of this code in no way implies    *
 * endorsement by the FDA or confers any advantage in regulatory decisions.     *
 * Although this software can be redistributed and/or modified freely, we       *
 * ask that any derivative works bear some notice that they are derived from    *
 * it, and any modified versions bear some notice that they have been modified. * 
 *                                                                              *
 * Written by:                                                                  *
 *  Lars Johannesen                                                             *
 ********************************************************************************/
#include "mex.h"

#include <ecglib>

#include "matconfig.hpp"

#include <stdexcept>

template<typename TYP>
TYP get_matlab_scalar(const mxArray *a) {
        TYP *t = (TYP*)mxGetData(a);
        return *t;
}

template<typename TYP>
mxArray *set_matlab_scalar(TYP t, mxClassID cla=mxDOUBLE_CLASS) {
        mwSize dims[2] = {1,1};
        mxArray *o = mxCreateNumericArray(2, dims, cla, mxREAL);
        double *d = (double*)mxGetData(o);
        *d=t;
        return o;
}

bool isDoubleScalar(const mxArray *in) {
        return mxIsDouble(in) && mxGetNumberOfElements(in)==1;
}

void mexFunction (int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	using namespace ecglib;

	qrsdetector_config qcfg;

	if(nrhs == 0 && nlhs <= 1) {
		cinc2014::configto_matlab cfgtomat;
		qcfg.save(cfgtomat);

		plhs[0] = cfgtomat.opts();
		return;
	}

	if(nrhs < 2)
		mexErrMsgTxt("Need at least 2 args: u3, fs, [cfg]");

	try {
		if(!mxIsDouble(prhs[0]))
			mexErrMsgTxt("First argument must be a matrix");

		if(!mxIsDouble(prhs[1]))
			mexErrMsgTxt("Second argument must be a double scalar");

		if(nrhs == 3) {
			cinc2014::configfrom_matlab cfgfrommat(prhs[2]);
			qcfg.load(cfgfrommat);
		}

		double *u3ptr = (double*)mxGetPr(prhs[0]);
		int cols = mxGetN(prhs[0]); // Cols
		int rows = mxGetM(prhs[0]); // Rows
		mat u3(u3ptr, rows, cols);

		double fs = get_matlab_scalar<double>(prhs[1]);

		rrpredictor rrpred(qcfg);
		qrsdetectorpolicy_debug<rrpredictor> qrspol(rrpred, cols, qcfg);
		qrsdetector<qrsdetectorpolicy_debug<rrpredictor>> det(cols, qcfg, qrspol);

		int ntrain = qcfg.get<int>("trainingtime");
		if(ntrain < 0) {
			ntrain = u3.n_rows;
		} else {
			ntrain = static_cast<int>(round(ntrain * fs));
			
			if (ntrain > u3.n_rows) {
				ntrain = u3.n_rows;
			}
		}
		det.train(u3, ntrain, u3.n_cols, fs);
		auto pm = det.detect(u3, u3.n_rows, u3.n_cols, fs);

		std::vector<annotation> locs;
		get_annotations(pm[ecglib::GLOBAL_LEAD], annotation_type::RPEAK, locs);
		mxArray *o = mxCreateDoubleMatrix(locs.size(), 1, mxREAL);
		double *op = (double*)mxGetPr(o);
		for(std::size_t i = 0; i < locs.size(); ++i) {
			op[i] = static_cast<double>(round(locs[i].location() * fs/1000.));
		}

		plhs[0] = o;

		if(nlhs > 1) {
			mwSize cdims[1];
			cdims[0] = cols;

			mxArray *c = mxCreateCellArray(1,cdims);
			for(std::size_t i = 0; i < cols; ++i) {
				mat dbg = det.policy().debug(i);

				mwSize adims[2];
				adims[0] = dbg.n_rows;
				adims[1] = dbg.n_cols;
				mxArray *a = mxCreateNumericArray(2, adims, mxDOUBLE_CLASS, mxREAL);
				double *dst = (double*)mxGetData(a);

				memcpy(dst, dbg.memptr(), dbg.n_rows * dbg.n_cols*sizeof(double));

				mxSetCell(c, i, a);
			}

			plhs[1] = c;
		}

		if(nlhs > 2) {
			cinc2014::configto_matlab cfgtomat;
			qcfg.save(cfgtomat);

			plhs[2] = cfgtomat.opts();
		}
	} catch (std::logic_error &exp) {
		mexErrMsgTxt(exp.what());
	} catch (...) {
		mexErrMsgTxt("Unknown error");
	}
}
